@@ -4,11 +4,11 @@
  * GChemPaint library
  * fragment.cc
  *
- * Copyright (C) 2002-2011 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2012 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -235,7 +235,7 @@ bool Fragment::OnChanged (bool save)
 			m_BeginAtom = start;
 			m_EndAtom = CurPos;
 			if (residue)
-				residue->SetResidue (r);
+				residue->SetResidue (r, sy);
 			else {
 				map<gcu::Atom*, gcu::Bond*>::iterator i;
 				Bond *pBond = (gcp::Bond*) m_Atom->GetFirstBond (i);
@@ -307,7 +307,7 @@ bool Fragment::OnChanged (bool save)
 		if (r) {
 			m_EndAtom = m_BeginAtom + strlen (sy);
 			if (residue)
-				residue->SetResidue (r);
+				residue->SetResidue (r, sy);
 			else {
 				map<gcu::Atom*, gcu::Bond*>::iterator i;
 				Bond *pBond = (gcp::Bond*) m_Atom->GetFirstBond (i);
@@ -356,7 +356,7 @@ bool Fragment::OnChanged (bool save)
 	pView->Update (this);
 	m_bLoading = false;
 	m_Valid = Invalid;
-	Window* pWin = pDoc->GetWindow ();
+	Window *pWin = static_cast < Window * > (pDoc->GetWindow ());
 	if (m_Atom->GetZ () || ((m_buf.length () == 0) && (m_Atom->GetBondsNumber () == 0))) {
 		if (!pDoc->GetReadOnly ()) {
 			pWin->ActivateActionWidget ("/MainMenu/FileMenu/Save", true);
@@ -1113,7 +1113,7 @@ Object* Fragment::GetAtomAt (double x, double y, G_GNUC_UNUSED double z)
 		m_BeginAtom = cur;
 		m_EndAtom = cur + strlen (sy);
 		if (residue)
-			residue->SetResidue (r);
+			residue->SetResidue (r, sy);
 		else {
 			map<gcu::Atom*, gcu::Bond*>::iterator i;
 			Bond *pBond = (gcp::Bond*) m_Atom->GetFirstBond (i);
@@ -1338,10 +1338,10 @@ bool Fragment::Validate ()
 		&& m_Atom->GetBondsNumber () == 0)
 		return true;
 	if (m_Atom->GetZ() == 0 || (dynamic_cast <FragmentResidue*> (m_Atom) && !((FragmentResidue*) m_Atom)->GetResidue ())) {
-		Document *pDoc = dynamic_cast<Document*> (GetDocument ());
+		Document *pDoc = dynamic_cast <Document*> (GetDocument ());
 		m_TextItem->SetSelectionBounds (m_BeginAtom, (m_EndAtom == m_BeginAtom)? m_EndAtom + 1: m_EndAtom);
 		GtkWidget* w = gtk_message_dialog_new (
-										GTK_WINDOW (pDoc->GetWindow ()->GetWindow ()),
+										GTK_WINDOW (static_cast < Window *> (pDoc->GetWindow ())->GetWindow ()),
 										GTK_DIALOG_DESTROY_WITH_PARENT,
 										GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
 										_("Invalid symbol."));
@@ -1370,7 +1370,7 @@ bool Fragment::Validate ()
 			Document *pDoc = dynamic_cast<Document*> (GetDocument ());
 			m_TextItem->SetSelectionBounds ((*it)->GetStartIndex (), (*it)->GetEndIndex ());
 			GtkWidget* w = gtk_message_dialog_new (
-											GTK_WINDOW (pDoc->GetWindow ()->GetWindow ()),
+											GTK_WINDOW (static_cast <Window * > (pDoc->GetWindow ())->GetWindow ()),
 											GTK_DIALOG_DESTROY_WITH_PARENT,
 											GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
 											_("Invalid charge."));
