@@ -1,14 +1,14 @@
 // -*- C++ -*-
 
-/* 
+/*
  * Gnome Chemistry Utils
- * gcu/application.cc 
+ * gcu/application.cc
  *
  * Copyright (C) 2005-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -25,11 +25,9 @@
 #include "config.h"
 #include "application.h"
 #include "cmd-context.h"
-#include "cmd-context-gtk.h"
 #include "document.h"
 #include "loader.h"
-#include "message.h"
-#include "ui-builder.h"
+#include "ui-manager.h"
 #include <gsf/gsf-input-gio.h>
 #include <gsf/gsf-output-gio.h>
 #include <gsf/gsf-input-memory.h>
@@ -54,38 +52,8 @@ namespace gcu
 GOConfNode *Application::m_ConfDir = NULL;
 
 static map<string, Application *> Apps;
-WindowState Application::DefaultWindowState = NormalWindowState;
 static Application *Default = NULL;
 
-class ApplicationPrivate
-{
-public:
-	static void MaximizeWindows ();
-	static void FullScreenWindows ();
-	static bool LoadDatabases (Application *app);
-};
-
-void ApplicationPrivate::MaximizeWindows ()
-{
-	Application::DefaultWindowState = MaximizedWindowState;
-}
-
-void ApplicationPrivate::FullScreenWindows ()
-{
-	Application::DefaultWindowState = FullScreenWindowState;
-}
-
-bool ApplicationPrivate::LoadDatabases (Application *app)
-{
-	return false;
-}
-static GOptionEntry options[] = 
-{
-  {"full-screen", 'F', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, (void *)ApplicationPrivate::MaximizeWindows, N_("Open new windows full screen"), NULL},
-  {"maximize", 'M', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, (void *) ApplicationPrivate::FullScreenWindows, N_("Maximize new windows"), NULL},
-  {NULL, 0, 0, G_OPTION_ARG_NONE, NULL, NULL, NULL }
-};
-
 Application::Application (string name, string datadir, char const *help_name, char const *icon_name, CmdContext *cc)
 {
 	if (m_ConfDir == NULL) {
@@ -119,11 +87,9 @@ Application::Application (string name, string datadir, char const *help_name, ch
 		first_call = false;
 	}
 	IconName = icon_name? icon_name: (help_name? help_name: Name.c_str ());
-	GdkScreen *screen = gdk_screen_get_default ();
-	m_ScreenResolution = (unsigned) rint (gdk_screen_get_width (screen) * 25.4 / gdk_screen_get_width_mm (screen));
+	m_ScreenResolution = 72;
 	m_ImageResolution = m_ScreenResolution;
 	m_ImageHeight = m_ImageWidth = 300;
-	m_RecentManager = gtk_recent_manager_get_default ();
 
 	// check supported pixbuf formats
 	GSList *formats = gdk_pixbuf_get_formats ();
@@ -140,11 +106,12 @@ Application::Application (string name, string datadir, char const *help_name, ch
 		l = l->next;
 	}
 	g_slist_free (formats);
-	RegisterOptions (options);
 	if (Default == NULL)
 		Default = this;
 	RegisterBabelType ("chemical/x-xyz", "xyz");
+#if 0
 	g_idle_add (reinterpret_cast <GSourceFunc> (ApplicationPrivate::LoadDatabases), this);
+#endif
 }
 
 Application::~Application ()
@@ -181,7 +148,7 @@ bool Application::HasHelp ()
 	GFile *file = g_file_new_for_uri (HelpFilename.c_str ());
 	bool exists = g_file_query_exists (file, NULL);
 	g_object_unref (file);
-	return exists;	
+	return exists;
 }
 
 void Application::SetCurDir (char const* dir)
@@ -194,73 +161,6 @@ void Application::SetCurDir (string const &dir)
 	CurDir = dir;
 }
 
-void Application::OnMail (char const *MailAddress)
-{
-	go_url_show (MailAddress);
-}
-
-void Application::ShowURI (string& uri)
-{
-	go_url_show (uri.c_str ());
-}
-
-void Application::OnLiveAssistance ()
-{
-	go_url_show ("irc://irc.gimp.net/gchemutils");
-}
-
-static void on_res_changed (GtkSpinButton *btn, Application *app)
-{
-	app->SetImageResolution (gtk_spin_button_get_value_as_int (btn));
-}
-
-static void on_width_changed (GtkSpinButton *btn, Application *app)
-{
-	app->SetImageWidth (gtk_spin_button_get_value_as_int (btn));
-}
-
-static void on_height_changed (GtkSpinButton *btn, Application *app)
-{
-	app->SetImageHeight (gtk_spin_button_get_value_as_int (btn));
-}
-
-static void on_transparency_changed (GtkToggleButton *btn, Application *app)
-{
-	app->SetTransparentBackground (gtk_toggle_button_get_active (btn));
-}
-
-GtkWidget *Application::GetImageResolutionWidget ()
-{
-	UIBuilder *builder = new UIBuilder (UIDIR"/image-resolution.ui", GETTEXT_PACKAGE);
-	GtkWidget *w = builder->GetWidget ("screen-lbl");
-	char *buf = g_strdup_printf (_("(screen resolution is %u)"), m_ScreenResolution);
-	gtk_label_set_text (GTK_LABEL (w), buf);
-	g_free (buf);
-	w = builder->GetWidget ("res-btn");
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON (w), m_ImageResolution);
-	g_signal_connect (G_OBJECT (w), "value-changed", G_CALLBACK (on_res_changed), this);
-	w = builder->GetWidget ("transparent-btn");
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), m_TransparentBackground);
-	g_signal_connect (G_OBJECT (w), "toggled", G_CALLBACK (on_transparency_changed), this);
-	w = builder->GetRefdWidget ("res-table");
-	delete builder;
-	return w;
-}
-
-GtkWidget *Application::GetImageSizeWidget ()
-{
-	UIBuilder *builder = new UIBuilder (UIDIR"/image-size.ui", GETTEXT_PACKAGE);
-	GtkWidget *w = builder->GetWidget ("width");
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON (w), m_ImageWidth);
-	g_signal_connect (G_OBJECT (w), "value-changed", G_CALLBACK (on_width_changed), this);
-	w = builder->GetWidget ("height");
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON (w), m_ImageHeight);
-	g_signal_connect (G_OBJECT (w), "value-changed", G_CALLBACK (on_height_changed), this);
-	w = builder->GetRefdWidget ("size-table");
-	delete builder;
-	return w;
-}
-
 char const *Application::GetPixbufTypeName (string& filename, char const *mime_type)
 {
 	GdkPixbufFormat *format = m_SupportedPixbufFormats[mime_type];
@@ -289,7 +189,7 @@ char const *Application::GetPixbufTypeName (string& filename, char const *mime_t
 void Application::RemoveDocument (Document *Doc)
 {
 	m_Docs.erase (Doc);
-	if (m_Docs.size () == 0 && gtk_main_level ())
+	if (m_Docs.size () == 0 && LoopRunning ())
 		NoMoreDocsEvent ();
 }
 
@@ -301,10 +201,11 @@ ContentType Application::Load (std::string const &uri, const gchar *mime_type, D
 		l = Loader::GetLoader ("chemical/x-cml");
 		if (!l)
 			return ContentTypeUnknown;
-		char *cml = ConvertToCML (uri, mime_type);
+		char *cml = ConvertToCML (uri, mime_type, options);
 		if (!cml)
 			return ContentTypeUnknown;
 		input = gsf_input_memory_new (const_cast <guint8 const *> (reinterpret_cast <guint8 *> (cml)), strlen (cml), true);
+		mime_type = "chemical/x-cml";
 	} else {
 		GError *error = NULL;
 		input = gsf_input_gio_new_for_uri (uri.c_str (), &error);
@@ -326,11 +227,23 @@ ContentType Application::Load (std::string const &uri, const gchar *mime_type, D
 ContentType Application::Load (GsfInput *input, const gchar *mime_type, Document* Doc, const char *options)
 {
 	Loader *l = Loader::GetLoader (mime_type);
-	if (!l)
-		return ContentTypeUnknown;
+	bool needs_free = false;
+	if (!l) {
+		l = Loader::GetLoader ("chemical/x-cml");
+		if (!l)
+			return ContentTypeUnknown;
+		char *cml = ConvertToCML (input, mime_type, options);
+		if (!cml)
+			return ContentTypeUnknown;
+		input = gsf_input_memory_new (const_cast <guint8 const *> (reinterpret_cast <guint8 *> (cml)), strlen (cml), true);
+		mime_type = "chemical/x-cml";
+		needs_free = true;
+	}
 	GOIOContext *io = GetCmdContext ()->GetNewGOIOContext ();
 	ContentType ret = l->Read (Doc, input, mime_type, io);
 	g_object_unref (io);
+	if (needs_free)
+		g_object_unref (input);
 	return ret;
 }
 
@@ -338,13 +251,21 @@ bool Application::Save (std::string const &uri, const gchar *mime_type, Object c
 {
 	Loader *l = Loader::GetSaver (mime_type);
 	GError *error = NULL;
+	GOIOContext *io = GetCmdContext ()->GetNewGOIOContext ();
 	if (!l) {
 		l = Loader::GetSaver ("chemical/x-cml");
-		if (!l)
+		if (!l) {
+			g_object_unref (io);
 			return false;
+		}
 		GsfOutput *output = gsf_output_memory_new ();
-		
-		return false;
+		l->Write (Obj, output, "chemical/x-cml", io, type);
+		guint8 const* cml = gsf_output_memory_get_bytes (GSF_OUTPUT_MEMORY (output));
+		g_object_unref (io);
+		if (cml)
+			ConvertFromCML (reinterpret_cast < char const * > (cml), uri, mime_type, options);
+		g_object_unref (output);
+		return true;
 	}
 	GFile *file = g_file_new_for_uri (uri.c_str ());
 	if (g_file_query_exists (file, NULL)) {
@@ -354,10 +275,9 @@ bool Application::Save (std::string const &uri, const gchar *mime_type, Object c
 			char *unescaped = g_uri_unescape_string (uri.c_str (), NULL);
 			ostringstream str;
 			str << _("Error while processing ") << unescaped << ":\n" << error->message;
-			Message *Box = new Message (this, str.str ().c_str (), GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE);
+			m_CmdContext->Message (str.str ().c_str (), CmdContext::SeverityError, false);
 			g_free (unescaped);
 			g_error_free (error);
-			Box->Run ();
 			g_object_unref (file);
 			return false;
 		}
@@ -367,7 +287,6 @@ bool Application::Save (std::string const &uri, const gchar *mime_type, Object c
 	if (error) {
 		g_error_free (error);
 	}
-	GOIOContext *io = GetCmdContext ()->GetNewGOIOContext ();
 	bool ret = l->Write (Obj, output, mime_type, io, type);
 	g_object_unref (output);
 	g_object_unref (io);
@@ -485,18 +404,6 @@ Object* Application::CreateObject (const std::string& TypeName, Object* parent)
 	return obj;
 }
 
-bool Application::BuildObjectContextualMenu (Object *target, GtkUIManager *UIManager, Object *object, double x, double y)
-{
-	bool result = false;
-	TypeDesc const *typedesc = target->m_TypeDesc;
-	if (!typedesc)
-		return false;
-	list<BuildMenuCb>::const_iterator i, end = typedesc->MenuCbs.end ();
-	for (i = typedesc->MenuCbs.begin (); i != end; i++)
-		result |= (*i) (target, UIManager, object, x, y);
-	return result;
-}
-
 void Application::AddRule (const string& type1, RuleId rule, const string& type2)
 {
 	AddRule (Object::GetTypeId (type1), rule, Object::GetTypeId (type2));
@@ -582,10 +489,22 @@ TypeDesc const *Application::GetTypeDescription (TypeId Id)
 	return (i != m_Types.end ())? &(*i).second: NULL;
 }
 
+bool Application::BuildObjectContextualMenu (Object *target, UIManager *uim, Object *object, double x, double y)
+{
+	bool result = false;
+	TypeDesc const *typedesc = target->m_TypeDesc;
+	if (!typedesc)
+		return false;
+	list<BuildMenuCb>::const_iterator i, end = typedesc->MenuCbs.end ();
+	for (i = typedesc->MenuCbs.begin (); i != end; i++)
+		result |= (*i) (target, uim, object, x, y);
+	return result;
+}
+
 CmdContext *Application::GetCmdContext ()
 {
 	if (m_CmdContext == NULL)
-		m_CmdContext = new CmdContextGtk (this);
+		CreateDefaultCmdContext ();
 	return m_CmdContext;
 }
 
@@ -632,7 +551,7 @@ int Application::OpenBabelSocket ()
 	adr_serv.sun_family = AF_UNIX;
 	strcpy (adr_serv.sun_path, socket_path.c_str ());
 	if (connect (res, (const struct sockaddr*) &adr_serv, sizeof (struct sockaddr_un)) == -1) {
-		perror ("Connexion failed");
+		perror (_("Connection failed"));
 		return -1;
 	}
 	return res;
@@ -642,7 +561,7 @@ int Application::OpenBabelSocket ()
 @param uri the uri of the document to convert.
 @param mime_type the mime type of the document.
 @param options options to pass to OpenBabel.
- 
+
 This method converts the source to CML.
 @return the converted text as a newly allocate string or NULL.
 */
@@ -747,22 +666,13 @@ ok_exit:
 	return start;
 }
 
-/*!
-@param input a source GsfInput.
-@param mime_type the mime type of the document.
-@param options options to pass to OpenBabel.
- 
-This method converts the source to CML.
-@return the converted text as a newly allocate string or NULL.
-*/
-
 char* Application::ConvertToCML (GsfInput *input, const char *mime_type, const char *options)
 {
 	int sock = OpenBabelSocket ();
 	if (sock <= 0)
 		return NULL;
 	size_t n = gsf_input_size (input);
-	char const *outbuf = reinterpret_cast <char const *> (gsf_input_read (input, n, NULL)); 
+	char const *outbuf = reinterpret_cast <char const *> (gsf_input_read (input, n, NULL));
 	std::string buf = "-i ";
 	buf += MimeToBabelType (mime_type);
 	buf += " -o cml";
@@ -805,14 +715,6 @@ ok_exit:
 	return start;
 }
 
-/*!
-@param cml: the CML string to convert.
-@param uri the uri of the document to which the document will be saved.
-@param mime_type the mime type of the document.
-@param options options to pass to OpenBabel.
- 
-This method converts CML to a target.
-*/
 void Application::ConvertFromCML (char const *cml, std::string const &uri, const char *mime_type, const char *options)
 {
 	int sock = OpenBabelSocket ();
@@ -870,14 +772,6 @@ exit_point:
 	close (sock);
 }
 
-/*!
-@param cml: the CML string to convert.
-@param output a target GsfOutput.
-@param mime_type the mime type of the document.
-@param options options to pass to OpenBabel.
- 
-This method converts CML to a target.
-*/
 void Application::ConvertFromCML (char const *cml, GsfOutput *output, const char *mime_type, const char *options)
 {
 	int sock = OpenBabelSocket ();
